@@ -3,15 +3,8 @@ use base qw(Froody::Implementation);
 
 use strict;
 use warnings;
-
-use Froody::Response::XML;
-use Froody::Response::Terse;
-
-
 use Params::Validate qw( :all );
 
-our $IMPLEMENTATION_SUPERCLASS = 1;
-
 sub implements { "Froody::API::Reflection" => 'froody.reflection.*' }
 
 use Froody::Logger;
@@ -36,25 +29,22 @@ Returns information for a given froody API method.
 
 sub getMethodInfo
 {
-  my ($self, $args, $metadata) = @_;
-  
-  my $method_name = $args->{method_name};
-  my $method = $metadata->{dispatcher}->get_method($method_name);
+  my $self = shift;
+  my $args = shift;
+  my $calling_method = shift;
 
-  return $self->_methodInfo($metadata, $method);
-}
+  my $repository = $self->repository;
 
-sub _methodInfo {
-  my ($self, $metadata, $method) = @_;
-  my $calling_method = $metadata->{dispatcher}->get_method('froody.reflection.getMethodInfo');
+  my $method = $repository->get_method($args->{method_name});
 
   my $response = {
     name => $method->full_name,
   };
 
-  $response->{description} = $method->description
-    if $method->description;
-  $response->{needslogin} = $method->needslogin || 0;
+  for (qw(needslogin description)) {
+    my $val = $method->$_;
+    $response->{$_} = $val if $val;
+  }
 
   my $arg_info;
   {
@@ -65,7 +55,7 @@ sub _methodInfo {
         name => $k,
         -text => $v->{doc},
         optional => $v->{optional},
-        type => join(',',@{$v->{type}}),
+        type => $v->{usertype}
       };
       push @$arg_info, $argdata;
     }
@@ -75,7 +65,7 @@ sub _methodInfo {
   my $method_errors = $method->errors;
   my $errors = [ map { 
     +{ 
-        code => $_,
+        code => $_, 
         message => $method_errors->{$_}{message}, 
         -text => $method_errors->{$_}{description} 
     } } keys %$method_errors ];
@@ -92,18 +82,25 @@ sub _methodInfo {
   # child nodes.  This *must* be encoded in what we are (which is utf-8)
   if ($method->example_response) {
     my ($example_element) = $rsp->xml->findnodes("//response");
-    $example_element->appendText( _response_to_xml($method)->toString );
+    $example_element->addChild( _response_to_xml($method) );
   }
-  return $rsp->as_terse; 
+
+  return $rsp; 
 }
 
+use Froody::Response::XML;
+use Froody::Response::PerlDS;
+
 sub _response_to_xml {
   my $structure = shift;
 
-  # convert whatever we have to XML -- example
-  # responses are always stored in terse form (for now)
+  # convert whatever we have to XML
   my $example = $structure->example_response->as_xml;
 
+  # Nasty hack incase things aren't in the right encoding
+  unless ($example->xml->encoding eq "utf-8")
+   { $example = $example->as_perlds->as_xml };
+ 
   # grab the thingy inside the rsp and return it
   my ($response) = $example->xml->findnodes("/rsp/*");
   return $response->cloneNode(1);
@@ -118,16 +115,13 @@ Returns a list of methods.
 
 sub getMethods
 {
-  my ($self, $args, $metadata) = @_;
+  my $self = shift;
+  my $args = shift;
+
+  my $repository =  $self->repository;
 
-  my $client = $metadata->{dispatcher};
-  my $repo = $metadata->{dispatcher}->repository;
-  my %methods = map { $_ => 1 } 
-                map { $_->full_name } 
-                $repo->get_methods;
-  
   return {
-    method => [ sort map { $_ } keys %methods],
+    method => [ sort map { $_->full_name } $repository->get_methods ],
   };
 }
 
@@ -139,13 +133,15 @@ Returns a list of error types.
 
 sub getErrorTypes
 {
-  my ($self, $args, $metadata) = @_;
+  my $self = shift;
+
+  my $repository = $self->repository;
 
   return {
-    errortype => [ sort
-                   grep { $_ }
+    errortype => [ sort 
                    map { $_->full_name } 
-                   $metadata->{dispatcher}->repository->get_errortypes ],
+                   grep { $_->full_name } 
+                   $repository->get_errortypes ],
   };
 }
 
@@ -157,51 +153,14 @@ Returns the error type information
 
 sub getErrorTypeInfo
 {
-  my ($self, $args, $metadata) = @_;
+  my $self = shift;
+  my $args = shift;
+  my $calling_method = shift;
 
-  my $repo = $metadata->{dispatcher}->repository;
-  my $et = $repo->get_errortype($args->{code});
-  return _errortype_hash($et->example_response); 
-}
+  my $repository = $self->repository;
+  my $et = $repository->get_errortype($args->{code});
 
-=item getSpecification($args, $metadata)
-
-Returns a terse representation of all public registered methods and error types
-within the repository indicated in $metadata
-
-=cut
-
-sub getSpecification {
-  my ($self, $args, $metadata) = @_;
-
-  my $repo = $metadata->{dispatcher}->repository;
-  my $methods = [ map { $self->_methodInfo($metadata, $_->[0])->as_terse->content }
-                  sort { $a->[1] cmp $b->[1] } 
-                  map { [$_, $_->full_name ] } 
-                  $repo->get_methods ];
-                  
-  my $errortypes = [ 
-                   map { _errortype_hash($_->[0]->example_response) }
-                   sort { $a->[1] cmp $b->[1] }
-                   grep { $_->[1] } # Skip the default structure
-                   map { [ $_, $_->full_name ] } 
-                   $repo->get_errortypes 
-                   ];
-  my $ret = {
-    methods => { method => $methods },
-    errortypes => { errortype => $errortypes },
-  };
-  return $ret;
-}
-
-sub _errortype_hash {
-  my $example = shift;
-  my $xml = $example->as_xml->xml; 
-  my $ret = {
-    code => $xml->findvalue('/rsp/err/@code'),
-    -text => join('', map { $_->toString } ($xml->findnodes('/rsp/err/*')))
-  };
-  return $ret;
+  return $et->example_response; 
 }
 
 =back
