@@ -28,14 +28,10 @@ sub new {
   my $self = $class->SUPER::new(@_);
 
   my $vars = $self->get_params;
+
   my $method = delete $vars->{method};
   $self->method($method);
 
-  my $type = delete $vars->{'_type'} || delete $vars->{'_froody_type'};
-  $self->type($type);
-  if ($type and $type eq 'json') {
-    $self->callback( delete $vars->{"_json_callback"} );
-  }
   $self->params($vars);
   
   return $self;
@@ -56,19 +52,22 @@ sub get_params {
   my %vars = map {
     my @results = $ar->param($_);
     @results = map { Encode::decode("utf-8", $_, 1) } @results;
-    ( Encode::decode("utf-8", $_, 1) => scalar(@results) > 1 ? [ @results ] : $results[0] );
-  } map {
-    ref($_) || s/\xef\xbb\xbf//; $_; # remove BOM from strings
+    ( $_ => scalar(@results) > 1 ? [ @results ] : $results[0] );
   } $ar->param();
 
   foreach my $upload ( $ar->upload ) {
     my $name = $upload->name;
-    $vars{$name} = -f $upload->tempname ? Froody::Upload
+    $vars{$name} = Froody::Upload
       ->new->fh($upload->fh)
            ->filename($upload->tempname)
            ->client_filename($upload->filename)
-           ->mime_type($upload->info->{'Content-Type'})
-    : undef;
+           ->mime_type($upload->info->{'Content-Type'});
+  }
+
+  # read cookies into the request as well.
+  for (CGI::cookie()) {
+    next unless $_ eq 'cookie_session';
+    $vars{$_} ||= CGI::cookie($_);
   }
 
   return \%vars;
